@@ -1,23 +1,18 @@
+#
 # Copyright (c) nexB Inc. and others. All rights reserved.
-# http://nexb.com and https://github.com/nexB/container-inspector/
+# SPDX-License-Identifier: Apache-2.0
+# See http://www.apache.org/licenses/LICENSE-2.0 for the license text.
+# See https://github.com/nexB/container-inspector for support or download.
+# See https://aboutcode.org for more information about nexB OSS projects.
 #
-# This software is licensed under the Apache License version 2.0.#
-#
-# You may not use this software except in compliance with the License.
-# You may obtain a copy of the License at:
-#     http://apache.org/licenses/LICENSE-2.0
-# Unless required by applicable law or agreed to in writing, software distributed
-# under the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
-# CONDITIONS OF ANY KIND, either express or implied. See the License for the
-# specific language governing permissions and limitations under the License.
 
-import csv as csv_module
-import json as json_module
 import logging
 import os
-from os import path
 import sys
 import tempfile
+import csv as csv_module
+import json as json_module
+from os import path
 
 import click
 
@@ -25,10 +20,11 @@ from container_inspector import image
 from container_inspector import dockerfile
 from container_inspector import rootfs
 
+TRACE = False
 logger = logging.getLogger(__name__)
-# un-comment these lines to enable logging
-# logging.basicConfig(level=logging.DEBUG, stream=sys.stdout)
-# logger.setLevel(logging.DEBUG)
+if TRACE:
+    logging.basicConfig(level=logging.DEBUG, stream=sys.stdout)
+    logger.setLevel(logging.DEBUG)
 
 
 @click.command()
@@ -99,17 +95,20 @@ def container_inspector(image_path, extract_to=None, csv=False):
     click.echo(results)
 
 
-def _container_inspector(image_path, extract_to=None, csv=False):
+def _container_inspector(image_path, extract_to=None, csv=False, _layer_path_segments=2):
     images = get_images_from_dir_or_tarball(image_path, extract_to=extract_to)
     as_json = not csv
 
     if as_json:
-        images = [i.to_dict() for i in images]
+        images = [i.to_dict(layer_path_segments=_layer_path_segments) for i in images]
         return json_module.dumps(images, indent=2)
     else:
         from io import StringIO
         output = StringIO()
-        flat = list(image.flatten_images_data(images))
+        flat = list(image.flatten_images_data(
+            images=images,
+            layer_path_segments=_layer_path_segments
+        ))
         if not flat:
             return
         keys = flat[0].keys()
