@@ -1,15 +1,10 @@
+#
 # Copyright (c) nexB Inc. and others. All rights reserved.
-# http://nexb.com and https://github.com/nexB/container-inspector/
+# SPDX-License-Identifier: Apache-2.0
+# See http://www.apache.org/licenses/LICENSE-2.0 for the license text.
+# See https://github.com/nexB/container-inspector for support or download.
+# See https://aboutcode.org for more information about nexB OSS projects.
 #
-# This software is licensed under the Apache License version 2.0.#
-#
-# You may not use this software except in compliance with the License.
-# You may obtain a copy of the License at:
-#     http://apache.org/licenses/LICENSE-2.0
-# Unless required by applicable law or agreed to in writing, software distributed
-# under the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
-# CONDITIONS OF ANY KIND, either express or implied. See the License for the
-# specific language governing permissions and limitations under the License.
 
 import logging
 import os
@@ -26,18 +21,16 @@ from container_inspector.utils import as_bare_id
 from container_inspector.utils import load_json
 from container_inspector.utils import sha256_digest
 
+TRACE = False
 logger = logging.getLogger(__name__)
-# un-comment these lines to enable logging
-# logging.basicConfig(level=logging.DEBUG, stream=sys.stdout)
-# logger.setLevel(logging.DEBUG)
-
-
-def logger_debug(*args):
-    return logger.debug(' '.join(isinstance(a, str) and a or repr(a) for a in args))
+if TRACE:
+    import sys
+    logging.basicConfig(level=logging.DEBUG, stream=sys.stdout)
+    logger.setLevel(logging.DEBUG)
 
 """
-Objects to handle Docker and OCI images and Layers.
-
+This module contains objects and utilities to handle Docker and OCI images and
+Layers.
 
 Supported formats:
 - docker v1.1 and v1.2
@@ -68,32 +61,23 @@ The OCI specs:
 """
 
 
-class ToDictMixin(object):
-    """
-    A mixin to add an to_dict() method to an attr-based class.
-    """
-
-    def to_dict(self, exclude_fields=()):
-        if exclude_fields:
-            filt = lambda attr, value: attr.name not in exclude_fields
-        else:
-            filt = lambda attr, value: True
-        return attr.asdict(self, filter=filt)
-
-
-def flatten_images_data(images):
+def flatten_images_data(images, layer_path_segments=0, _test=False):
     """
     Yield mapping for each layer of each image of an `images` list of Image.
     This is a flat data structure for CSV and tabular output.
+    Keep only ``layer_path_segments`` trailing layer location segments (or keep
+    the locations unmodified if ``layer_path_segments`` is 0)
     """
 
     for img in images:
+        img_extracted_location = img.extracted_location
         base_data = dict(
-            image_extracted_location=img.extracted_location,
-            image_archive_location=img.archive_location,
+            image_extracted_location='' if _test else img_extracted_location,
+            image_archive_location='' if _test else img.archive_location,
             image_id=img.image_id,
             image_tags=','.join(img.tags),
         )
+
         for layer in img.layers:
             layer_data = dict(base_data)
             layer_data['is_empty_layer'] = layer.is_empty_layer
@@ -103,11 +87,48 @@ def flatten_images_data(images):
             layer_data['created_by'] = layer.created_by
             layer_data['created'] = layer.created
             layer_data['comment'] = layer.comment
-            layer_data['layer_extracted_location'] = layer.extracted_location
-            layer_data['layer_archive_location'] = layer.archive_location
+
+            lay_extracted_location = layer.extracted_location
+            lay_archive_location = layer.archive_location
+
+            if layer_path_segments:
+                lay_extracted_location = get_trimmed_path(
+                    location=lay_extracted_location,
+                    num_segments=layer_path_segments,
+                )
+                lay_archive_location = get_trimmed_path(
+                    location=lay_archive_location,
+                    num_segments=layer_path_segments,
+                )
+
+            layer_data['layer_archive_location'] = lay_archive_location
+            layer_data['layer_extracted_location'] = lay_extracted_location
             yield layer_data
 
 
+def get_trimmed_path(location, num_segments=2):
+    """
+    Return a trimmed relative path given a location keeping only the
+    ``num_segments`` trailing path segments.
+
+    For example::
+    >>> assert get_trimmed_path(None) == None
+    >>> assert get_trimmed_path('a/b/c') == 'b/c'
+    >>> assert get_trimmed_path('/b/c') == 'b/c'
+    >>> assert get_trimmed_path('b/c') == 'b/c'
+    >>> assert get_trimmed_path('b/c/') == 'b/c/'
+    >>> assert get_trimmed_path('/x/a/b/c/', 3) == 'a/b/c/'
+    >>> assert get_trimmed_path('/x/a/b/c', 3) == 'a/b/c'
+    """
+    if location:
+        ends = location.endswith('/')
+        segments = location.strip('/').split('/')[-num_segments:]
+        relative = '/'.join(segments)
+        if ends:
+            relative += '/'
+        return relative
+
+
 @attr.attributes
 class ConfigMixin(object):
     """
@@ -204,7 +225,7 @@ class ArchiveMixin:
     archive_location = attr.attrib(
         default=None,
         metadata=dict(doc=
-            'Absolute directory location of this Archive original archive.'
+            'Absolute directory location of this Archive original file.'
             'May be empty if this was created from an extracted_location directory.'
         )
     )
@@ -224,12 +245,13 @@ class ArchiveMixin:
 
 
 @attr.attributes
-class Image(ArchiveMixin, ConfigMixin, ToDictMixin):
+class Image(ArchiveMixin, ConfigMixin):
     """
     A container image with pointers to its layers.
     Image objects can be created from these inputs:
     - an image tarball in docker format (e.g. "docker save").
     - a directory that contains an extracted image tarball in these layouts.
+
     OCI format is not yet supported.
     """
 
@@ -290,6 +312,36 @@ class Image(ArchiveMixin, ConfigMixin, ToDictMixin):
         if not self.image_format:
             self.image_format = self.find_format(self.extracted_location)
 
+    def to_dict(self, layer_path_segments=0, _test=False):
+        """
+        Return a dictionary of this image fields, excluding ``exclude_fields``.
+        Keep only ``layer_path_segments`` trailing layer location segments (or
+        keep the locations unmodified if ``layer_path_segments`` is 0).
+        """
+        image = attr.asdict(self)
+
+        if layer_path_segments:
+            for layer in image.get('layers', []):
+                layer['extracted_location'] = get_trimmed_path(
+                    location=layer.get('extracted_location'),
+                    num_segments=layer_path_segments,
+                )
+
+                layer['archive_location'] = get_trimmed_path(
+                    location=layer.get('archive_location'),
+                    num_segments=layer_path_segments,
+                )
+
+        if _test:
+            image['extracted_location'] = ''
+            img_archive_location = self.archive_location
+            image['archive_location'] = (
+                img_archive_location
+                and os.path.basename(img_archive_location)
+                or ''
+            )
+        return image
+
     @property
     def top_layer(self):
         """
@@ -304,14 +356,23 @@ class Image(ArchiveMixin, ConfigMixin, ToDictMixin):
         """
         return self.layers[0]
 
-    def extract_layers(self, extracted_location):
+    def extract_layers(self, extracted_location, as_events=False, skip_symlinks=True):
         """
         Extract all layer archives to the `extracted_location` directory.
         Each layer is extracted to its own directory named after its `layer_id`.
+        Skip symlinks and links if ``skip_symlinks`` is True.
+        Return a list of ExtractEvent if ``as_events`` is True or a list of message strings otherwise.
         """
+        all_events = []
         for layer in self.layers:
             exloc = os.path.join(extracted_location, layer.layer_id)
-            layer.extract(extracted_location=exloc)
+            events = layer.extract(
+                extracted_location=exloc,
+                skip_symlinks=skip_symlinks,
+                as_events=as_events,
+            )
+            all_events.extend(events)
+        return events
 
     def get_layers_resources(self, with_dir=False):
         """
@@ -398,14 +459,18 @@ class Image(ArchiveMixin, ConfigMixin, ToDictMixin):
                 yield purl, package, layer
 
     @staticmethod
-    def extract(archive_location, extracted_location):
+    def extract(archive_location, extracted_location, as_events=False, skip_symlinks=False):
         """
         Extract the image archive tarball at ``archive_location`` to
         ``extracted_location``.
+        Skip symlinks and links if ``skip_symlinks`` is True.
+        Return a list of ExtractEvent if ``as_events`` is True or a list of message strings otherwise.
         """
-        utils.extract_tar_keeping_symlinks(
+        return utils.extract_tar(
             location=archive_location,
             target_dir=extracted_location,
+            skip_symlinks=skip_symlinks,
+            as_events=as_events,
         )
 
     @staticmethod
@@ -413,19 +478,34 @@ class Image(ArchiveMixin, ConfigMixin, ToDictMixin):
         archive_location,
         extracted_location,
         verify=True,
+        skip_symlinks=False,
     ):
         """
-        Return a list of Images found in the tarball at `archive_location` that
-        will be extracted to `extracted_location`. The tarball must be in the
+        Return a list of Images found in the tarball at ``archive_location`` that
+        will be extracted to ``extracted_location``. The tarball must be in the
         format of a "docker save" command tarball.
 
-        If `verify` is True, perform extra checks on the config data and layers
+        If ``verify`` is True, perform extra checks on the config data and layers
         checksums.
+        Skip symlinks and links if ``skip_symlinks`` is True.
+        Ignore the extract events from extraction.
         """
-        Image.extract(
+        if TRACE:
+            logger.debug(
+                f'get_images_from_tarball: {archive_location} '
+                f'extracting to: {extracted_location}'
+            )
+
+        # TODO: do not ignore extract events
+        _events = Image.extract(
             archive_location=archive_location,
             extracted_location=extracted_location,
+            skip_symlinks=skip_symlinks,
         )
+        if TRACE:
+            logger.debug(f'get_images_from_tarball: events')
+            for e in _events:
+                logger.debug(str(e))
 
         return Image.get_images_from_dir(
             extracted_location=extracted_location,
@@ -446,10 +526,19 @@ class Image(ArchiveMixin, ConfigMixin, ToDictMixin):
         If `verify` is True, perform extra checks on the config data and layers
         checksums.
         """
+        if TRACE:
+            logger.debug(
+                f'get_images_from_dir: from  {extracted_location} and '
+                f'archive_location: {archive_location}',
+            )
+
         if not os.path.isdir(extracted_location):
             raise Exception(f'Not a directory: {extracted_location}')
 
         image_format = Image.find_format(extracted_location)
+
+        if TRACE: logger.debug(f'get_images_from_dir: image_format: {image_format}')
+
         if image_format == 'docker':
             return Image.get_docker_images_from_dir(
                 extracted_location=extracted_location,
@@ -457,7 +546,7 @@ class Image(ArchiveMixin, ConfigMixin, ToDictMixin):
                 verify=verify,
         )
 
-        if image_format == 'docker':
+        if image_format == 'oci':
             return Image.get_oci_images_from_dir(
                 extracted_location=extracted_location,
                 archive_location=archive_location,
@@ -506,6 +595,9 @@ class Image(ArchiveMixin, ConfigMixin, ToDictMixin):
             ....
         ]
         """
+        if TRACE:
+            logger.debug(f'get_docker_images_from_dir: {extracted_location}')
+
         if not os.path.isdir(extracted_location):
             raise Exception(f'Not a directory: {extracted_location}')
 
@@ -517,15 +609,23 @@ class Image(ArchiveMixin, ConfigMixin, ToDictMixin):
 
         manifest = load_json(manifest_loc)
 
+        if TRACE:
+            logger.debug(f'get_docker_images_from_dir: manifest: {manifest}')
+
         images = []
         for manifest_config in manifest:
-            images.append(
-                Image.from_docker_manifest_config(
-                    extracted_location=extracted_location,
-                    archive_location=archive_location,
-                    manifest_config=manifest_config,
-                    verify=verify,
-            ))
+            if TRACE:
+                logger.debug(f'get_docker_images_from_dir: manifest_config: {manifest_config}')
+            img = Image.from_docker_manifest_config(
+                extracted_location=extracted_location,
+                archive_location=archive_location,
+                manifest_config=manifest_config,
+                verify=verify,
+
+            )
+            if TRACE: logger.debug(f'get_docker_images_from_dir: img: {img!r}')
+
+            images.append(img)
 
         return images
 
@@ -600,6 +700,8 @@ class Image(ArchiveMixin, ConfigMixin, ToDictMixin):
             }
          }
         """
+        if TRACE: logger.debug(f'from_docker_manifest_config: manifest_config: {manifest_config!r}')
+
         manifest_config = utils.lower_keys(manifest_config)
 
         config_file = manifest_config.get('config') or ''
@@ -627,7 +729,9 @@ class Image(ArchiveMixin, ConfigMixin, ToDictMixin):
 
         layer_paths = manifest_config.get('layers') or []
         layers_archive_locs = [
-            os.path.join(extracted_location, lp) for lp in layer_paths]
+            os.path.join(extracted_location, lp)
+            for lp in layer_paths
+        ]
 
         tags = manifest_config.get('repotags') or []
 
@@ -689,7 +793,7 @@ class Image(ArchiveMixin, ConfigMixin, ToDictMixin):
         - oci (which is for the OCI format)
         """
         clue_files_by_image_format = {
-            'docker': ('manifest.json', 'repositories',),
+            'docker': ('manifest.json',),
             'oci': ('blobs', 'index.json', 'oci-layout',)
          }
 
@@ -901,7 +1005,7 @@ def assign_history_to_layers(history, layers):
 
 
 @attr.attributes
-class Resource(ToDictMixin):
+class Resource:
     path = attr.attrib(
         default=None,
         metadata=dict(doc='Rootfs-relative path for this Resource.')
@@ -929,9 +1033,12 @@ class Resource(ToDictMixin):
         metadata=dict(doc='True for symlink.')
     )
 
+    def to_dict(self, **kwargs):
+        return attr.asdict(self)
+
 
 @attr.attributes
-class Layer(ArchiveMixin, ConfigMixin, ToDictMixin):
+class Layer(ArchiveMixin, ConfigMixin):
     """
     A layer object represents a slice of a root filesystem in a container image.
     """
@@ -985,15 +1092,19 @@ class Layer(ArchiveMixin, ConfigMixin, ToDictMixin):
         if not self.size:
             self.size = os.path.getsize(self.archive_location)
 
-    def extract(self, extracted_location):
+    def extract(self, extracted_location, as_events=False, skip_symlinks=False):
         """
         Extract this layer archive in the `extracted_location` directory and set
         this Layer ``extracted_location`` attribute to ``extracted_location``.
+        Skip symlinks and links if ``skip_symlinks`` is True.
+        Return a list of ExtractEvent if ``as_events`` is True or a list of message strings otherwise.
         """
         self.extracted_location = extracted_location
-        utils.extract_tar(
+        return utils.extract_tar(
             location=self.archive_location,
             target_dir=extracted_location,
+            skip_symlinks=skip_symlinks,
+            as_events=as_events,
         )
 
     def get_resources(self, with_dir=False, walker=os.walk):
@@ -1022,7 +1133,7 @@ class Layer(ArchiveMixin, ConfigMixin, ToDictMixin):
                 yield build_resource(top, f, _is_file=True)
             if with_dir:
                 for d in dirs:
-                    yield build_resource(top, d, _is_file=True)
+                    yield build_resource(top, d, _is_file=False)
 
     def get_installed_packages(self, packages_getter):
         """
@@ -1041,3 +1152,6 @@ class Layer(ArchiveMixin, ConfigMixin, ToDictMixin):
           the same structure).
         """
         return packages_getter(self.extracted_location)
+
+    def to_dict(self, **kwargs):
+        return attr.asdict(self)
