@@ -1,15 +1,10 @@
+#
 # Copyright (c) nexB Inc. and others. All rights reserved.
-# http://nexb.com and https://github.com/nexB/container-inspector/
+# SPDX-License-Identifier: Apache-2.0
+# See http://www.apache.org/licenses/LICENSE-2.0 for the license text.
+# See https://github.com/nexB/container-inspector for support or download.
+# See https://aboutcode.org for more information about nexB OSS projects.
 #
-# This software is licensed under the Apache License version 2.0.#
-#
-# You may not use this software except in compliance with the License.
-# You may obtain a copy of the License at:
-#     http://apache.org/licenses/LICENSE-2.0
-# Unless required by applicable law or agreed to in writing, software distributed
-# under the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
-# CONDITIONS OF ANY KIND, either express or implied. See the License for the
-# specific language governing permissions and limitations under the License.
 
 import logging
 import os
@@ -17,12 +12,14 @@ import tempfile
 
 from commoncode.fileutils import copytree
 from commoncode.fileutils import delete
+from commoncode.paths import split
 
+TRACE = False
 logger = logging.getLogger(__name__)
-# un-comment these lines to enable logging
-# import sys
-# logging.basicConfig(level=logging.DEBUG, stream=sys.stdout)
-# logger.setLevel(logging.DEBUG)
+if TRACE:
+    import sys
+    logging.basicConfig(level=logging.DEBUG, stream=sys.stdout)
+    logger.setLevel(logging.DEBUG)
 
 """
 Utilities to handle image and layer archives and recreate proper rootfs
@@ -34,7 +31,7 @@ class InconsistentLayersError(Exception):
     pass
 
 
-def rebuild_rootfs(img, target_dir):
+def rebuild_rootfs(img, target_dir, skip_symlinks=True):
     """
     Extract and merge or "squash" all layers of the `image` Image in a single
     rootfs in `target_dir`. Extraction is done in sequence from the bottom (root
@@ -42,6 +39,8 @@ def rebuild_rootfs(img, target_dir):
     unionfs/overlayfs procedure is applied at each step as per the OCI spec:
     https://github.com/opencontainers/image-spec/blob/master/layer.md#whiteouts
 
+    Skip symlinks and links if ``skip_symlinks`` is True.
+
     Return a list of deleted "whiteout" files.
     Raise an Exception on errrors.
 
@@ -68,7 +67,7 @@ def rebuild_rootfs(img, target_dir):
     deletions = []
 
     for layer_num, layer in enumerate(img.layers):
-        logger.debug(
+        if TRACE: logger.debug(
             f'Extracting layer {layer_num} - {layer.layer_id} '
             f'tarball: {layer.archive_location}'
         )
@@ -76,17 +75,24 @@ def rebuild_rootfs(img, target_dir):
         # 1. extract a layer to temp.
         # Note that we are not preserving any special file and any file permission
         extracted_loc = tempfile.mkdtemp('container_inspector-docker')
-        layer.extract(extracted_location=extracted_loc)
-        logger.debug(f'  Extracted layer to: {extracted_loc}')
+        # TODO: do not ignore extract events
+        _events = layer.extract(
+            extracted_location=extracted_loc,
+            skip_symlinks=skip_symlinks,
+        )
+        if TRACE:
+            logger.debug(f'  Extracted layer to: {extracted_loc} with skip_symlinks: {skip_symlinks}')
+            for ev in _events:
+                logger.debug(f'  {ev}')
 
         # 2. find whiteouts in that layer.
         whiteouts = list(find_whiteouts(extracted_loc))
-        logger.debug('  Merging extracted layers and applying unionfs whiteouts')
-        logger.debug('  Whiteouts:\n' + '     \n'.join(map(repr, whiteouts)))
+        if TRACE: logger.debug('  Merging extracted layers and applying unionfs whiteouts')
+        if TRACE: logger.debug('  Whiteouts:\n' + '     \n'.join(map(repr, whiteouts)))
 
         # 3. remove whiteouts in the previous layer stack (e.g. the WIP rootfs)
         for whiteout_marker_loc, whiteable_path in whiteouts:
-            logger.debug(f'    Deleting dir or file with whiteout marker: {whiteout_marker_loc}')
+            if TRACE: logger.debug(f'    Deleting dir or file with whiteout marker: {whiteout_marker_loc}')
             whiteable_loc = os.path.join(target_dir, whiteable_path)
             delete(whiteable_loc)
             # also delete the whiteout marker file
@@ -94,21 +100,22 @@ def rebuild_rootfs(img, target_dir):
             deletions.append(whiteable_loc)
 
         # 4. finall copy/overwrite the extracted layer over the WIP rootfs
-        logger.debug(f'  Moving extracted layer from: {extracted_loc} to: {target_dir}')
+        if TRACE: logger.debug(f'  Moving extracted layer from: {extracted_loc} to: {target_dir}')
         copytree(extracted_loc, target_dir)
-        logger.debug(f'  Moved layer to: {target_dir}')
+        if TRACE: logger.debug(f'  Moved layer to: {target_dir}')
         delete(extracted_loc)
 
     return deletions
 
 
-WHITEOUT_EXPLICIT_PREFIX = '.wh.'
-WHITEOUT_OPAQUE_PREFIX = '.wh..wh.opq'
+WHITEOUT_PREFIX = '.wh.'
+WHITEOUT_SPECIAL_PREFIX = '.wh..wh'
+WHITEOUT_OPAQUE_PREFIX = '.wh..wh..opq'
 
 
-def is_whiteout_marker(path):
+def is_whiteout_marker(file_name):
     """
-    Return True if the ``path`` is a whiteout marker file.
+    Return True if the ``file_name`` is a whiteout marker file.
 
     For example::
     >>> is_whiteout_marker('.wh.somepath')
@@ -117,32 +124,80 @@ def is_whiteout_marker(path):
     True
     >>> is_whiteout_marker('somepath.wh.')
     False
-    >>> is_whiteout_marker('somepath/.wh.foo')
-    True
-    >>> is_whiteout_marker('somepath/.wh.foo/')
+    >>> is_whiteout_marker('.wh.foo')
     True
     """
-    file_name = path and os.path.basename(path.strip('/')) or ''
-    return file_name.startswith(WHITEOUT_EXPLICIT_PREFIX)
+    return file_name and file_name.startswith(WHITEOUT_PREFIX)
+
+
+def is_whiteout_opaque_marker(file_name):
+    """
+    Return True if the ``file_name`` is an opaque whiteout marker file.
+
+    For example::
+    >>> is_whiteout_opaque_marker('.wh.somepath')
+    False
+    >>> is_whiteout_opaque_marker('.wh..wh.opq')
+    False
+    >>> is_whiteout_opaque_marker('.wh..wh..opq')
+    True
+    >>> is_whiteout_opaque_marker('somepath..wh..wh..opq')
+    False
+    >>> is_whiteout_opaque_marker('.wh..wh.plnk')
+    False
+    >>> is_whiteout_opaque_marker('.wh..wh..opq.foo')
+    False
+    >>> is_whiteout_opaque_marker('somepath/.wh..wh..opq/')
+    False
+    """
+    return file_name and file_name == WHITEOUT_OPAQUE_PREFIX
+
+
+def is_whiteout_special_marker(file_name):
+    """
+    Return True if the ``file_name`` is an opaque whiteout marker file.
+
+    For example::
+    >>> is_whiteout_special_marker('.wh.somepath')
+    False
+    >>> is_whiteout_special_marker('.wh..wh.opq')
+    True
+    >>> is_whiteout_special_marker('.wh..wh..opq')
+    True
+    >>> is_whiteout_special_marker('.wh..wh.plnk')
+    True
+    >>> is_whiteout_special_marker('somepath..wh..wh..opq')
+    False
+    >>> is_whiteout_special_marker('.wh..wh..opq.foo')
+    True
+    >>> is_whiteout_special_marker('somepath/.wh..wh..opq/')
+    False
+    """
+    return file_name and file_name.startswith(WHITEOUT_SPECIAL_PREFIX)
 
 
 def get_whiteable_path(path):
     """
     Return the whiteable path for ``path`` or None if this not a whiteable path.
-    TODO: Handle OSses with case-insensitive FS (e.g. Windows)
     """
+    # FIXME: Handle OSses with case-insensitive FS (e.g. Windows)
     file_name = os.path.basename(path)
     parent_dir = os.path.dirname(path)
 
-    if file_name == WHITEOUT_OPAQUE_PREFIX:
+    if is_whiteout_special_marker(file_name):
         # Opaque whiteouts means the whole parent directory should be removed
         # https://github.com/opencontainers/image-spec/blob/master/layer.md#whiteouts
+        # note as a simplification we treat all of these as opaque, and log these that are not
+        if not is_whiteout_opaque_marker(file_name):
+            # This is the case for legacy AUFS '.wh..wh.plnk' and '.wh..wh.aufs'
+            # only seen in legacy Docker
+            logger.error(f'ERROR: unsupported whiteout filename: {file_name}')
         return parent_dir
 
-    if file_name.startswith(WHITEOUT_EXPLICIT_PREFIX):
+    elif is_whiteout_marker(file_name):
         # Explicit, file-only whiteout
         # https://github.com/opencontainers/image-spec/blob/master/layer.md#whiteouts
-        _, _, real_file_name = file_name.rpartition(WHITEOUT_EXPLICIT_PREFIX)
+        _, _, real_file_name = file_name.rpartition(WHITEOUT_PREFIX)
         return os.path.join(parent_dir, real_file_name)
 
 
@@ -193,6 +248,24 @@ WINDOWS_PATHS = set([
 ])
 
 
+def compute_path_depth(root_path, dir_path):
+    """
+    Compute the depth of ``dir_path`` below ``root_path`` as the number of paths
+    segments that extend below the root.
+    """
+    if not dir_path:
+        return 0
+    dir_path = dir_path.strip('/')
+
+    if not root_path:
+        return len(split(dir_path))
+
+    root_path = root_path.strip('/')
+
+    suffix = dir_path[len(root_path):]
+    return len(split(suffix))
+
+
 def find_root(
     location,
     max_depth=3,
@@ -202,18 +275,38 @@ def find_root(
 ):
     """
     Return the first likely location of the root of a filesystem found in the
-    `location` directory and looking down up to `max_depth` directory levels
-    deep below the location directory. If `max_depth` == 0, look at full depth.
-    Search for well known directories listed in the `root_paths` set. A root
-    directory is return as found if at least `min_paths` exists as filenames or
-    directories under it.
+    ``location`` directory and below up and including to ``max_depth`` directory
+    levels deep below the ``location`` root directory.
 
-    `walker` is a callable that behaves the same as `os.walk() and is used
-    for testing`
+    If ``max_depth`` == 0, look at full depth.
+
+    Search for well known directories listed in the ``root_paths`` set. A root
+    directory is returned as found if at least ``min_paths`` exists as filenames
+    or directories under it.
+
+    ``walker`` is a callable behaving like ``os.walk()`` and is used for testing.
     """
-    for depth, (top, dirs, files) in enumerate(walker(location), 1):
+    if TRACE: logger.debug(
+        f'find_root: location={location!r}, max_depth={max_depth!r}, '
+        f'root_paths={root_paths!r}, min_paths={min_paths!r}'
+    )
+    depth = 0
+    for top, dirs, files in walker(location):
+        if TRACE: logger.debug(f' find_root: top={top!r}, dirs={dirs!r}, files={files!r}')
+        if max_depth:
+            depth = compute_path_depth(location, top)
+            if TRACE: logger.debug(f'  find_root: top depth={depth!r}')
+            if depth > max_depth:
+                if TRACE: logger.debug(
+                    f'    find_root: max_depth={max_depth!r}, '
+                    f'depth={depth!r} returning None')
+                return
+
         matches = len(set(dirs + files) & root_paths)
+        if TRACE: logger.debug(f'  find_root: top={top!r}, matches={matches!r}')
+
         if matches >= min_paths:
+            if TRACE: logger.debug(f'    find_root: matches >= min_paths: returning {top!r}')
             return top
-        if max_depth and depth == max_depth:
-            return
+
+    if TRACE: logger.debug(f'find_root: noting found: returning None')
