@@ -1,15 +1,10 @@
+#
 # Copyright (c) nexB Inc. and others. All rights reserved.
-# http://nexb.com and https://github.com/nexB/container-inspector/
+# SPDX-License-Identifier: Apache-2.0
+# See http://www.apache.org/licenses/LICENSE-2.0 for the license text.
+# See https://github.com/nexB/container-inspector for support or download.
+# See https://aboutcode.org for more information about nexB OSS projects.
 #
-# This software is licensed under the Apache License version 2.0.#
-#
-# You may not use this software except in compliance with the License.
-# You may obtain a copy of the License at:
-#     http://apache.org/licenses/LICENSE-2.0
-# Unless required by applicable law or agreed to in writing, software distributed
-# under the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
-# CONDITIONS OF ANY KIND, either express or implied. See the License for the
-# specific language governing permissions and limitations under the License.
 
 import os
 import json
@@ -24,17 +19,24 @@ from utilities import check_expected
 
 def clean_images_data(images):
     """
-    Clean an Image.to_dict() for testing
+    Clean an a list of Image.to_dict() for testing
     """
-    for i in images:
-        i['extracted_location'] = None
-        i['archive_location'] = None
-        for l in i['layers']:
-            l['extracted_location'] = None
-            l['archive_location'] = None
+    for image in images:
+        clean_image_data(image)
     return images
 
 
+def clean_image_data(image):
+    """
+    Clean `image` data from Image.to_dict() for testing
+    """
+    image['extracted_location'] = ''
+    image['archive_location'] = os.path.basename(image['archive_location'] or '')
+    
+    return image
+
+
+
 class TestContainerInspectorCli(FileBasedTesting):
     test_data_dir = os.path.join(os.path.dirname(__file__), 'data')
 
@@ -48,14 +50,14 @@ class TestContainerInspectorCli(FileBasedTesting):
     def test_container_inspector_single_layer_from_tarball(self):
         test_dir = self.get_test_loc('cli/hello-world.tar')
         expected = self.get_test_loc('cli/hello-world.tar-inventory-from-tarball-expected.json')
-        out = cli._container_inspector(image_path=test_dir)
+        out = cli._container_inspector(image_path=test_dir, _layer_path_segments=1)
         result = clean_images_data(json.loads(out))
         check_expected(result, expected, regen=False)
 
     def test_container_inspector_multiple_layers_from_tarball(self):
         test_dir = self.get_test_loc('cli/she-image_from_scratch-1.0.tar')
         expected = self.get_test_loc('cli/she-image_from_scratch-1.0.tar-inventory-from-tarball-expected.json')
-        out = cli._container_inspector(image_path=test_dir)
+        out = cli._container_inspector(image_path=test_dir, _layer_path_segments=1)
         result = clean_images_data(json.loads(out))
         check_expected(result, expected, regen=False)
 
