@@ -1,15 +1,10 @@
+#
 # Copyright (c) nexB Inc. and others. All rights reserved.
-# http://nexb.com and https://github.com/nexB/container-inspector/
+# SPDX-License-Identifier: Apache-2.0
+# See http://www.apache.org/licenses/LICENSE-2.0 for the license text.
+# See https://github.com/nexB/container-inspector for support or download.
+# See https://aboutcode.org for more information about nexB OSS projects.
 #
-# This software is licensed under the Apache License version 2.0.#
-#
-# You may not use this software except in compliance with the License.
-# You may obtain a copy of the License at:
-#     http://apache.org/licenses/LICENSE-2.0
-# Unless required by applicable law or agreed to in writing, software distributed
-# under the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
-# CONDITIONS OF ANY KIND, either express or implied. See the License for the
-# specific language governing permissions and limitations under the License.
 
 import os
 
@@ -210,7 +205,7 @@ class TestRootfs(testcase.FileBasedTesting):
         assert not rootfs.find_root(test_dir, max_depth=1)
         assert not rootfs.find_root(test_dir, max_depth=2)
         assert not rootfs.find_root(test_dir, max_depth=3)
-        assert not rootfs.find_root(test_dir, max_depth=4)
+        assert rootfs.find_root(test_dir, max_depth=4).endswith('level1/level2/level3')
 
         expected = '/find_root/level1/level2/level3'
         found = rootfs.find_root(test_dir, max_depth=5)
@@ -221,3 +216,16 @@ class TestRootfs(testcase.FileBasedTesting):
 
         found = rootfs.find_root(os.path.join(test_dir, 'find_root'), max_depth=4)
         assert found.replace(test_dir, '') == expected
+
+    def test_rootfs_compute_path_depth(self):
+        assert rootfs.compute_path_depth(None, None) == 0
+        assert rootfs.compute_path_depth('', '') == 0
+        assert rootfs.compute_path_depth(None, 'foo') == 1
+        assert rootfs.compute_path_depth('foo', None) == 0
+        assert rootfs.compute_path_depth('/root', '/root/find_root') == 1
+        assert rootfs.compute_path_depth('/root', '/root/one/2/') == 2
+        assert rootfs.compute_path_depth('/root/', '/root/one/2/') == 2
+        assert rootfs.compute_path_depth('root/', '/root/one/2') == 2
+        assert rootfs.compute_path_depth('root/', '/root/') == 0
+        assert rootfs.compute_path_depth('root/', '/root/') == 0
+        assert rootfs.compute_path_depth('root/', '/root/1/2/3/4') == 4
